import sys
import libxml2
import os
import re
import logging
import logging.config
from logging.handlers import RotatingFileHandler

from gnosis.xml.objectify import _XO_
from gnosis.xml.objectify import *

from stacklet.task.stacktask import *


PARSER='EXPAT'

class BuildContext:
    def __init__(self):
        self.counter = 0;
        self._commandDict = {}
        self._propertiesDict = {}

    def get_all_properties(self):
        self._propertiesDict.update(self._commandDict)
        return self._propertiesDict

    def get_global_asset_dir(self):
        return self.inputDocAbsoluteFilePath + '/../../assets/'

    def get_asset_dir(self):
        return self.inputDocAbsoluteFilePath + '/' + self.get_all_properties()['_build.assetLoc_'] + '/'

def _init_logging(buildContext):

    loggerConfig = ''
    if buildContext._commandDict.has_key('_build.loggerConfig_'):
        loggerConfig = buildContext._commandDict['_build.loggerConfig_']
    else:
        #look for logger.conf in the directory containing this module
        factory_path = os.path.dirname(os.path.realpath( __file__ ))
        loggerConfig = os.path.join(factory_path, 'logger.conf')

    msg = ''
    try:
        if os.access(loggerConfig, os.F_OK):
            logging.config.fileConfig(loggerConfig)
            buildContext.logger = logging.getLogger("outLogger")
            for h in buildContext.logger.handlers:
                if isinstance(h, RotatingFileHandler) and h.backupCount > 0:
                    h.doRollover()
            buildContext.logger.info('Loaded logging config ' + loggerConfig)
            return
        else:
            msg = 'Log config file ' + loggerConfig + ' not found'
    except:
            msg = 'Error parsing log config file ' + loggerConfig + ' ' + str(sys.exc_info()[0])

    #if we fall through to here there was a problem loading the logger
    #config, so setup some basic logging parameters
    buildContext.logger = logging.getLogger('root')
    buildContext.logger.setLevel(logging.INFO)
    ch = logging.StreamHandler()
    ch.setLevel(logging.INFO)
    formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
    ch.setFormatter(formatter)
    buildContext.logger.addHandler(ch)
    buildContext.logger.warning(msg + '.  Using default logging parameters.')


def _parse_input_file(buildContext, xmlInput):

    buildContext.logger.info('Parsing ' + xmlInput)
    buildContext.inputDoc = libxml2.parseFile(xmlInput)
    buildContext.inputDocAbsoluteFilePath = os.path.dirname(os.path.abspath(buildContext.inputDoc.name))
    buildContext.inputDoc.xincludeProcess()

def _process_imports(buildContext, xmlInput):

    try:
        imports = buildContext.inputDoc.xpathEval("//imports")[0].content
    except IndexError:
        buildContext.logger.info('stackbuilder: no imports found')
        return

    for module in imports.split():
        buildContext.logger.info('Importing ' + module)
        __import__(module, globals(), locals())

def _validate_namespace(py_obj):
    VALID_NS = ['http://stacklet.com/specfile/v1']
    ns = ''
    if hasattr(py_obj, 'xmlns:stk'):
        ns = getattr(py_obj, 'xmlns:stk')
        if (ns in VALID_NS):
            return
    raise ValueError('Invalid namespace declaration ' + ns + '.  Valid namespace(s): ' + str(VALID_NS))


def process_input_file(buildContext, xmlInput):
    try:
        _parse_input_file(buildContext, xmlInput)
    except:
        buildContext.logger.exception('stackbuilder error: Parsing input file ' + xmlInput + ' failed')
        sys.exit(1)

    try:
        _process_imports(buildContext, xmlInput)
    except:
        buildContext.logger.exception('stackbuilder error: Processing imports in input file ' + xmlInput + ' failed')
        sys.exit(1)

    try:
        py_obj = XML_Objectify(str(buildContext.inputDoc)).make_instance()
        _validate_namespace(py_obj)
    except:
        buildContext.logger.exception('stackbuilder error: XML_Objectify on input file ' + xmlInput + ' failed')
        sys.exit(1)

    try:
        for child in children(py_obj):
            if isinstance(child, StackTask):
                child.process(buildContext)
        buildContext.logger.info('Done processing ' + xmlInput)
    except:
        buildContext.logger.exception('stackbuilder error: Processing tasks in input file ' + xmlInput + ' failed')
        sys.exit(1)

def main():
    buildContext = BuildContext()
    _XO_._seq = None #workaround to a bug in objectify v1.2.2
    if len(sys.argv) <= 1:
        sys.exit('stackbuilder error: No input file was specified\n')

    dict = {}
    inputFiles = list()
    p = re.compile('^_.*_=.*')
    for i in range(1, len(sys.argv)):
        arg = sys.argv[i]
        if p.search(arg):
            propName, propVal = arg.split('=', 1)[0:]
            dict[propName] = propVal
        else:
            inputFiles.append(arg)
    buildContext._commandDict = dict
    _init_logging(buildContext)
    for xmlInput in inputFiles:
        process_input_file(buildContext, xmlInput)
    buildContext.logger.info('Exiting build.')



if __name__ == "__main__":
    main()
