"""Tasks for manipulating consoles.

Please see docstring of each class for usage.

"""
import os

from gnosis.xml.objectify import _XO_
from gnosis.xml.objectify import *

from stacklet.task.stacktask import *
from stacklet.task.stackerror import *
from stacklet.util import fileutil

class _XO_console(_XO_, StackTask):
    """Add a console to /etc/inittab and /etc/securetty in the image under construction

    Task Arguments:
    name -- name of the console

    """
    _TASK_NAME = 'console'

    def __init__(self):
        pass

    def process(self, buildContext):
        try:
            try:
                self._buildContext = buildContext
                self.log_info(buildContext, 'entering')
                self.precheck(buildContext)
                cmds = list()
                if os.access(buildContext.mountPoint + '/etc/inittab', os.F_OK):
                    line = self._name + ':' + self._init + ' ' + self._name
                    fileutil.append_lines(buildContext, line, '/etc/inittab', '^' + self._name + ':')
                if os.access(buildContext.mountPoint + '/etc/securetty', os.F_OK):
                    line = self._name
                    fileutil.append_lines(buildContext, self._name, '/etc/securetty', '^' + self._name + '$')

                self.exec_command_chroot(buildContext, cmds)
            except TaskError, taskErr:
                if taskErr.criticality == TaskError.ABORT_BUILD:
                    raise
                self.log_error(buildContext, taskErr)
        finally:
            self.log_info(buildContext, 'leaving')

    def onerr(self, buildContext, cmd, retval):
        print 'ignoring ' + str(retval)

    def precheck(self, buildContext):
        self.is_valid_build_context(buildContext)

        self._name = self.get_value('name')

        if self.is_empty(self._name):
            raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'name is required')

        self._init = self.get_value('init')

        if self.is_empty(self._init):
            self._init = '2345:respawn:/sbin/mingetty'

gnosis.xml.objectify._XO_console = _XO_console
gnosis.xml.objectify._XO_stk_console = _XO_console