"""Tasks for creating devices.

Please see docstring of each class for usage.

"""
from gnosis.xml.objectify import _XO_
from gnosis.xml.objectify import *

from stacklet.task.stacktask import *
from stacklet.task.stackerror import *

class _XO_device(_XO_, StackTask):
    """Create a device file in /dev in image under construction.

    Task Arguments:
    name -- the name of the device file without path info
    mode -- the mode of the created device file (optional)
    spec -- space-separated triplet: type major minor
    """
    _TASK_NAME = 'device'

    def __init__(self):
        pass

    def process(self, buildContext):
        try:
            try:
                self._buildContext = buildContext
                self.log_info(buildContext, 'entering')
                self.precheck(buildContext)
                cmds = list()

                modeArg = ''
                if not self.is_empty(self._mode):
                    modeArg = '-m ' + self._mode

                cmd = 'mknod ' +  modeArg + ' ' + buildContext.mountPoint + '/dev/' + self._name + ' ' + self._spec
                self.exec_command(buildContext, cmd)
            except TaskError, taskErr:
                if taskErr.criticality == TaskError.ABORT_BUILD:
                    raise
                self.log_error(buildContext, taskErr)
        finally:
            self.log_info(buildContext, 'leaving')

    def precheck(self, buildContext):
        self.is_valid_build_context(buildContext)

        self._name = self.get_value('name')
        self._mode = self.get_value('mode')
        self._spec = self.get_value('spec')

        if self.is_empty(self._name):
            raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'name is required')

        if self.is_empty(self._spec):
            raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'spec is required')


gnosis.xml.objectify._XO_device = _XO_device
gnosis.xml.objectify._XO_stk_device = _XO_device
