"""Tasks for loading properties into the build context.

Please see docstring of each class for usage.

"""
from gnosis.xml.objectify import _XO_
from gnosis.xml.objectify import *

from stacklet.task.stacktask import *
from stacklet.task.stackerror import *

class _XO_property(_XO_, StackTask):
    """Load values into the build context.  If the name is already
    assigned the new value will overwrite the old value.

    Task Arguments:
    name -- the name of the property
    value -- (optional) the value of the property.  Default is empty string
    """
    _TASK_NAME = 'property'

    def __init__(self):
        pass

    def process(self, buildContext):
        try:
            try:
                self._buildContext = buildContext
                self.log_info(buildContext, 'entering ')
                self.precheck(buildContext)

                dict = {}
                dict[self._name] = self._value
                buildContext._propertiesDict.update(dict)

            except TaskError, taskErr:
                if taskErr.criticality == TaskError.ABORT_BUILD:
                    raise
                self.log_error(buildContext, taskErr)
        finally:
            self.log_info(buildContext, 'leaving')

    def precheck(self, buildContext):

        self._name = self.get_value('name', substituteProperties=False)
        self._value = self.get_value('value', substituteProperties=False)

        if self.is_empty(self._name):
            raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'name is required')
        elif not (self._name.startswith('_') and self._name.endswith('_')):
            raise FieldError(self._TASK_NAME, TaskError.ABORT_TASK, 'name must start and end with underscore; encountered ' + self._name)

        if self.is_empty(self._value):
            self._value = ''

class _XO_properties(_XO_, StackTask):
    """Load values into the build context.  This is the container for
    property nodes.
    """
    _TASK_NAME = 'properties'

    def __init__(self):
        pass

    def process(self, buildContext):
        try:
            try:
                self._buildContext = buildContext
                self.log_info(buildContext, 'entering')
                self.precheck(buildContext)

                for child in children(self):
                    if isinstance(child, _XO_property):
                        child.process(buildContext)

            except TaskError, taskErr:
                if taskErr.criticality == TaskError.ABORT_BUILD:
                    raise
                self.log_error(buildContext, taskErr)
        finally:
            self.log_info(buildContext, 'leaving')

    def precheck(self, buildContext):
        pass

gnosis.xml.objectify._XO_property = _XO_property
gnosis.xml.objectify._XO_stk_property = _XO_property
gnosis.xml.objectify._XO_properties = _XO_properties
gnosis.xml.objectify._XO_stk_properties = _XO_properties

