"""Contains all exception classes thrown by Tasks

"""
class TaskError(Exception):
    """The base class for all task exceptions.

    Three levels of criticality are defined.  An exception
    that should stop the build entirely should be encoded
    with ABORT_BUILD.  An error that should stop execution
    of the task but allow processing to continue should
    be encoded with ABORT_TASK

    """
    WARNING = 'WARNING'
    ABORT_TASK = 'ABORT_TASK'
    ABORT_BUILD = 'ABORT_BUILD'

    def __init__(self):
        pass

    def __str__(self):
        return self.criticality + ' in <' + self.taskName + '>: ' + self.message

class FieldError(TaskError):
    """Field validation errors """
    def __init__(self, taskName, criticality, message):
        self.taskName = taskName
        self.criticality = criticality
        self.message = message

class ExecError(TaskError):
    """Command execution errors """
    def __init__(self, taskName, criticality, message):
        self.taskName = taskName
        self.criticality = criticality
        self.message = message

class BuildError(TaskError):
    """Misc build errors, eg. image is not mounted """
    def __init__(self, taskName, criticality, message):
        self.taskName = taskName
        self.criticality = criticality
        self.message = message
