"""File utilities.
"""
import os
import re
import types

def append_lines(buildContext, lines, file, expr):
    """Append lines to a file inside the image.

    arguments:
    buildContext -- the build context
    lines -- a string or list of strings to append (newlines are added automatically)
    file -- the name of the file to append to, path is relative to buildContext.mountPoint
    expr -- (optional) if the regex pattern exists in the file do *not* append (used to avoid dupe lines)
    """
    if isinstance(lines, types.StringTypes):
        lines = [lines]

    if not expr is None:
        if contains_expression(buildContext, file, expr):
            return

    for line in lines:
        f = open(buildContext.mountPoint + file, 'a')
        f.write(line +  "\n")
        f.close()

def contains_expression(buildContext, file, expr):
    """Search for a regex inside a file inside the image.  Return True if found.

    arguments:
    buildContext -- the build context
    file -- the name of the file to search, path is relative to buildContext.mountPoint
    expr -- the regex pattern to search for
    """
    f = None
    p = re.compile(expr)
    try:
        if os.access(buildContext.mountPoint + file, os.F_OK):
            f = open(buildContext.mountPoint + file, 'r')
            for line in f:
                if p.search(line):
                    return True
    finally:
        if not f is None:
            f.close()
    return False

